# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Any, List, Optional
from uuid import UUID

from tikka.domains.entities.category import Category
from tikka.interfaces.adapters.repository.categories import (
    CategoriesRepositoryInterface,
)
from tikka.interfaces.adapters.repository.db_repository import DBRepositoryInterface

TABLE_NAME = "categories"


class DBCategoriesRepository(CategoriesRepositoryInterface, DBRepositoryInterface):
    """
    DBCategoriesRepository class
    """

    def add(self, category: Category) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.add.__doc__
        )

        # insert only non hidden fields
        self.client.insert(
            TABLE_NAME,
            **get_fields_from_category(category),
        )

    def get(self, id: UUID) -> Optional[Category]:  # pylint: disable=redefined-builtin
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.get.__doc__
        )

        row = self.client.select_one(
            f"SELECT * FROM {TABLE_NAME} WHERE id=?", (id.hex,)
        )
        if row is None:
            return None

        return get_category_from_row(row)

    def update(self, category: Category) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.update.__doc__
        )

        # update only non hidden fields
        self.client.update(
            TABLE_NAME,
            f"id='{category.id.hex}'",
            **get_fields_from_category(category),
        )

    def update_parent_id(self, parent_id: UUID, new_parent_id: Optional[UUID]):
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.update_parent_id.__doc__
        )
        self.client.update(
            TABLE_NAME,
            f"parent_id='{parent_id.hex}'",
            parent_id=new_parent_id.hex if new_parent_id is not None else None,
        )

    def delete(self, id: UUID) -> None:  # pylint: disable=redefined-builtin
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.delete.__doc__
        )

        self.client.delete(TABLE_NAME, id=id.hex)

    def delete_all(self) -> None:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.delete_all.__doc__
        )

        self.client.clear(TABLE_NAME)

    def list(self, parent_id: Optional[UUID]) -> List[Category]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.list.__doc__
        )
        sql_parent_id = None if parent_id is None else parent_id.hex
        if sql_parent_id is None:
            sql = f"SELECT * FROM {TABLE_NAME} WHERE parent_id IS ? ORDER BY name ASC"
        else:
            sql = f"SELECT * FROM {TABLE_NAME} WHERE parent_id=? ORDER BY name ASC"
        result_set = self.client.select(
            sql,
            (sql_parent_id,),
        )

        list_ = []
        for row in result_set:
            list_.append(get_category_from_row(row))

        return list_

    def list_all(self) -> List[Category]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            CategoriesRepositoryInterface.list.__doc__
        )
        result_set = self.client.select(f"SELECT * FROM {TABLE_NAME} ORDER BY name ASC")

        list_ = []
        for row in result_set:
            list_.append(get_category_from_row(row))

        return list_


def get_fields_from_category(category: Category) -> dict:
    """
    Return a dict of supported fields with normalized value

    :param category: Category instance
    :return:
    """
    fields = {}
    for (key, value) in category.__dict__.items():
        if key.startswith("_"):
            continue
        if isinstance(value, UUID):
            value = value.hex
        elif isinstance(value, bool):
            value = 1 if value is True else 0
        fields[key] = value

    return fields


def get_category_from_row(row: tuple) -> Category:
    """
    Return a Category instance from a result set row

    :param row: Result set row
    :return:
    """
    values: List[Any] = []
    count = 0
    for value in row:
        if count in (0, 3):
            if value is not None:
                values.append(UUID(hex=value))
        elif count == 2:
            values.append(value == 1)
        else:
            values.append(value)
        count += 1

    return Category(*values)  # pylint: disable=no-value-for-parameter



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import annotations

import logging

from tikka.adapters.network.node.accounts import NodeAccounts
from tikka.adapters.network.node.authorities import NodeAuthorities
from tikka.adapters.network.node.connection import NodeConnection
from tikka.adapters.network.node.currency import NodeCurrency
from tikka.adapters.network.node.identities import NodeIdentities
from tikka.adapters.network.node.smiths import NodeSmiths
from tikka.adapters.network.node.technical_committee import NodeTechnicalCommittee
from tikka.adapters.network.node.transfers import NodeTransfers
from tikka.domains.entities.node import Node
from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.node.node import (
    NetworkNodeException,
    NetworkNodeInterface,
)


class NetworkNode(NetworkNodeInterface):
    """
    NetworkNode class
    """

    def __init__(self):
        """
        Init NetworkNode instance
        """
        self._connection = NodeConnection()
        self._accounts = NodeAccounts(self)
        self._authorities = NodeAuthorities(self)
        self._currency = NodeCurrency(self)
        self._identities = NodeIdentities(self)
        self._smiths = NodeSmiths(self)
        self._technical_committee = NodeTechnicalCommittee(self)
        self._transfers = NodeTransfers(self)

    @property
    def connection(self) -> NodeConnection:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkNodeInterface.connection.__doc__
        )
        return self._connection

    def get(self) -> Node:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkNodeInterface.get.__doc__
        )
        if not self.connection.is_connected() or self.connection.client is None:
            raise NetworkNodeException(NetworkConnectionError())

        try:
            response = self.connection.client.rpc_request("system_localPeerId", [])
        except Exception as exception:
            logging.exception(exception)
            raise NetworkNodeException(exception)

        if response is None:
            message = "node.get system_localPeerId returns None"
            logging.error(message)
            raise NetworkNodeException(message)

        peer_id = response.get("result")

        try:
            response = self.connection.client.rpc_request(  # type: ignore
                "system_syncState", []
            )
        except Exception as exception:
            logging.exception(exception)
            raise NetworkNodeException(exception)

        if response is None:
            message = "node.get system_syncState returns None"
            logging.error(message)
            raise NetworkNodeException(message)

        sync_state = response.get("result")
        current_block = sync_state["currentBlock"] if sync_state is not None else None

        try:
            current_epoch_result = self.connection.client.query("Babe", "EpochIndex")
        except Exception as exception:
            logging.exception(exception)
            raise NetworkNodeException(exception)

        try:
            response = self.connection.client.rpc_request("system_name", [])
        except Exception as exception:
            logging.exception(exception)
            raise NetworkNodeException(exception)

        if response is None:
            message = "node.get system_name returns None"
            logging.error(message)
            raise NetworkNodeException(message)

        chain_name = response.get("result")

        try:
            response = self.connection.client.rpc_request("system_version", [])
        except Exception as exception:
            logging.exception(exception)
            raise NetworkNodeException(exception)

        if response is None:
            message = "node.get system_version returns None"
            logging.error(message)
            raise NetworkNodeException(message)

        chain_version = response.get("result")

        unsafe_api_exposed = True
        try:
            self.connection.client.rpc_request("babe_epochAuthorship", [])
        except Exception as exception:
            error_message = str(exception)
            if "'code': -32601" not in error_message:
                logging.exception(exception)
            unsafe_api_exposed = False

        return Node(
            self.connection.client.url,
            peer_id=peer_id,
            block=current_block,
            software=chain_name,
            software_version=chain_version,
            epoch_index=current_epoch_result,
            unsafe_api_exposed=unsafe_api_exposed,
        )

    def get_ss58_prefix(self) -> int:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            NetworkNodeInterface.get.__doc__
        )
        if self.connection.client is not None:
            return self.connection.client.runtime_config.ss58_format

        return 0

    @property
    def accounts(self) -> NodeAccounts:
        """
        Return NodeAccountsInterface instance

        :return:
        """
        return self._accounts

    @property
    def authorities(self) -> NodeAuthorities:
        """
        Return NodeAuthoritiesInterface instance

        :return:
        """
        return self._authorities

    @property
    def currency(self) -> NodeCurrency:
        """
        Return NodeCurrencyInterface instance

        :return:
        """
        return self._currency

    @property
    def identities(self) -> NodeIdentities:
        """
        Return NodeIdentitiesInterface instance

        :return:
        """
        return self._identities

    @property
    def smiths(self) -> NodeSmiths:
        """
        Return NodeSmithsInterface instance

        :return:
        """
        return self._smiths

    @property
    def technical_committee(self) -> NodeTechnicalCommittee:
        """
        Return NodeTechnicalCommitteeInterface instance

        :return:
        """
        return self._technical_committee

    @property
    def transfers(self) -> NodeTransfers:
        """
        Return NodeTransfersInterface instance

        :return:
        """
        return self._transfers



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/currency.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_currencyWidget(object):
    def setupUi(self, currencyWidget):
        currencyWidget.setObjectName("currencyWidget")
        currencyWidget.resize(748, 719)
        self.verticalLayout = QtWidgets.QVBoxLayout(currencyWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.refreshButton = QtWidgets.QPushButton(currencyWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.verticalLayout.addWidget(self.refreshButton, 0, QtCore.Qt.AlignRight)
        self.errorLabel = QtWidgets.QLabel(currencyWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorLabel.sizePolicy().hasHeightForWidth())
        self.errorLabel.setSizePolicy(sizePolicy)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.scrollArea = QtWidgets.QScrollArea(currencyWidget)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, -376, 714, 1019))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.currencyGroupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.currencyGroupBox.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.currencyGroupBox.setObjectName("currencyGroupBox")
        self.formLayout_2 = QtWidgets.QFormLayout(self.currencyGroupBox)
        self.formLayout_2.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_2.setObjectName("formLayout_2")
        self.nameLabel = QtWidgets.QLabel(self.currencyGroupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.nameLabel.setFont(font)
        self.nameLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nameLabel.setObjectName("nameLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.nameLabel)
        self.nameValueLabel = QtWidgets.QLabel(self.currencyGroupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nameValueLabel.setFont(font)
        self.nameValueLabel.setText("")
        self.nameValueLabel.setObjectName("nameValueLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.nameValueLabel)
        self.symbolLabel = QtWidgets.QLabel(self.currencyGroupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.symbolLabel.setFont(font)
        self.symbolLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.symbolLabel.setObjectName("symbolLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.symbolLabel)
        self.symbolValueLabel = QtWidgets.QLabel(self.currencyGroupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.symbolValueLabel.setFont(font)
        self.symbolValueLabel.setText("")
        self.symbolValueLabel.setObjectName("symbolValueLabel")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.symbolValueLabel)
        self.universalDividendLabel = QtWidgets.QLabel(self.currencyGroupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.universalDividendLabel.setFont(font)
        self.universalDividendLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.universalDividendLabel.setObjectName("universalDividendLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.universalDividendLabel)
        self.universalDividendValueLabel = QtWidgets.QLabel(self.currencyGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.universalDividendValueLabel.sizePolicy().hasHeightForWidth())
        self.universalDividendValueLabel.setSizePolicy(sizePolicy)
        self.universalDividendValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.universalDividendValueLabel.setObjectName("universalDividendValueLabel")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.universalDividendValueLabel)
        self.monetaryMassLabel = QtWidgets.QLabel(self.currencyGroupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.monetaryMassLabel.setFont(font)
        self.monetaryMassLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.monetaryMassLabel.setObjectName("monetaryMassLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.monetaryMassLabel)
        self.monetaryMassValueLabel = QtWidgets.QLabel(self.currencyGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.monetaryMassValueLabel.sizePolicy().hasHeightForWidth())
        self.monetaryMassValueLabel.setSizePolicy(sizePolicy)
        self.monetaryMassValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.monetaryMassValueLabel.setObjectName("monetaryMassValueLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.monetaryMassValueLabel)
        self.membersLabel = QtWidgets.QLabel(self.currencyGroupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.membersLabel.setFont(font)
        self.membersLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.membersLabel.setObjectName("membersLabel")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.membersLabel)
        self.membersValueLabel = QtWidgets.QLabel(self.currencyGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.membersValueLabel.sizePolicy().hasHeightForWidth())
        self.membersValueLabel.setSizePolicy(sizePolicy)
        self.membersValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.membersValueLabel.setObjectName("membersValueLabel")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.membersValueLabel)
        self.verticalLayout_3.addWidget(self.currencyGroupBox)
        self.blockchainGroupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.blockchainGroupBox.setObjectName("blockchainGroupBox")
        self.formLayout_3 = QtWidgets.QFormLayout(self.blockchainGroupBox)
        self.formLayout_3.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_3.setObjectName("formLayout_3")
        self.expectedBlockDurationLabel = QtWidgets.QLabel(self.blockchainGroupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.expectedBlockDurationLabel.setFont(font)
        self.expectedBlockDurationLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.expectedBlockDurationLabel.setObjectName("expectedBlockDurationLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.expectedBlockDurationLabel)
        self.expectedBlockDurationValueLabel = QtWidgets.QLabel(self.blockchainGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.expectedBlockDurationValueLabel.sizePolicy().hasHeightForWidth())
        self.expectedBlockDurationValueLabel.setSizePolicy(sizePolicy)
        self.expectedBlockDurationValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.expectedBlockDurationValueLabel.setObjectName("expectedBlockDurationValueLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.expectedBlockDurationValueLabel)
        self.expectedEpochDurationLabel = QtWidgets.QLabel(self.blockchainGroupBox)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.expectedEpochDurationLabel.setFont(font)
        self.expectedEpochDurationLabel.setObjectName("expectedEpochDurationLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.expectedEpochDurationLabel)
        self.expectedEpochDurationValueLabel = QtWidgets.QLabel(self.blockchainGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.expectedEpochDurationValueLabel.sizePolicy().hasHeightForWidth())
        self.expectedEpochDurationValueLabel.setSizePolicy(sizePolicy)
        self.expectedEpochDurationValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.expectedEpochDurationValueLabel.setObjectName("expectedEpochDurationValueLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.expectedEpochDurationValueLabel)
        self.verticalLayout_3.addWidget(self.blockchainGroupBox)
        self.WOTGroupBox = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.WOTGroupBox.setObjectName("WOTGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.WOTGroupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.identityGroupBox = QtWidgets.QGroupBox(self.WOTGroupBox)
        self.identityGroupBox.setObjectName("identityGroupBox")
        self.formLayout_5 = QtWidgets.QFormLayout(self.identityGroupBox)
        self.formLayout_5.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_5.setFormAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.formLayout_5.setObjectName("formLayout_5")
        self.automaticRevocationPeriodLabel = QtWidgets.QLabel(self.identityGroupBox)
        self.automaticRevocationPeriodLabel.setObjectName("automaticRevocationPeriodLabel")
        self.formLayout_5.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.automaticRevocationPeriodLabel)
        self.automaticRevocationPeriodValueLabel = QtWidgets.QLabel(self.identityGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.automaticRevocationPeriodValueLabel.sizePolicy().hasHeightForWidth())
        self.automaticRevocationPeriodValueLabel.setSizePolicy(sizePolicy)
        self.automaticRevocationPeriodValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.automaticRevocationPeriodValueLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.automaticRevocationPeriodValueLabel.setObjectName("automaticRevocationPeriodValueLabel")
        self.formLayout_5.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.automaticRevocationPeriodValueLabel)
        self.minimumDelayBetweenChangingIdentityOwnerAccountLabel = QtWidgets.QLabel(self.identityGroupBox)
        self.minimumDelayBetweenChangingIdentityOwnerAccountLabel.setObjectName("minimumDelayBetweenChangingIdentityOwnerAccountLabel")
        self.formLayout_5.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.minimumDelayBetweenChangingIdentityOwnerAccountLabel)
        self.minimumDelayBetweenChangingIdentityOwnerAccountValueLabel = QtWidgets.QLabel(self.identityGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.minimumDelayBetweenChangingIdentityOwnerAccountValueLabel.sizePolicy().hasHeightForWidth())
        self.minimumDelayBetweenChangingIdentityOwnerAccountValueLabel.setSizePolicy(sizePolicy)
        self.minimumDelayBetweenChangingIdentityOwnerAccountValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.minimumDelayBetweenChangingIdentityOwnerAccountValueLabel.setObjectName("minimumDelayBetweenChangingIdentityOwnerAccountValueLabel")
        self.formLayout_5.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.minimumDelayBetweenChangingIdentityOwnerAccountValueLabel)
        self.confirmIdentityPeriodLabel = QtWidgets.QLabel(self.identityGroupBox)
        self.confirmIdentityPeriodLabel.setObjectName("confirmIdentityPeriodLabel")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.confirmIdentityPeriodLabel)
        self.confirmIdentityPeriodValueLabel = QtWidgets.QLabel(self.identityGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confirmIdentityPeriodValueLabel.sizePolicy().hasHeightForWidth())
        self.confirmIdentityPeriodValueLabel.setSizePolicy(sizePolicy)
        self.confirmIdentityPeriodValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.confirmIdentityPeriodValueLabel.setObjectName("confirmIdentityPeriodValueLabel")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.confirmIdentityPeriodValueLabel)
        self.identityValidationPeriodLabel = QtWidgets.QLabel(self.identityGroupBox)
        self.identityValidationPeriodLabel.setObjectName("identityValidationPeriodLabel")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.identityValidationPeriodLabel)
        self.identityValidationPeriodValueLabel = QtWidgets.QLabel(self.identityGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityValidationPeriodValueLabel.sizePolicy().hasHeightForWidth())
        self.identityValidationPeriodValueLabel.setSizePolicy(sizePolicy)
        self.identityValidationPeriodValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.identityValidationPeriodValueLabel.setObjectName("identityValidationPeriodValueLabel")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.identityValidationPeriodValueLabel)
        self.identityDeletionAfterRevocationLabel = QtWidgets.QLabel(self.identityGroupBox)
        self.identityDeletionAfterRevocationLabel.setObjectName("identityDeletionAfterRevocationLabel")
        self.formLayout_5.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.identityDeletionAfterRevocationLabel)
        self.identityDeletionAfterRevocationValueLabel = QtWidgets.QLabel(self.identityGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityDeletionAfterRevocationValueLabel.sizePolicy().hasHeightForWidth())
        self.identityDeletionAfterRevocationValueLabel.setSizePolicy(sizePolicy)
        self.identityDeletionAfterRevocationValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.identityDeletionAfterRevocationValueLabel.setObjectName("identityDeletionAfterRevocationValueLabel")
        self.formLayout_5.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.identityDeletionAfterRevocationValueLabel)
        self.minimumDelayBetweenIdentityCreationLabel = QtWidgets.QLabel(self.identityGroupBox)
        self.minimumDelayBetweenIdentityCreationLabel.setObjectName("minimumDelayBetweenIdentityCreationLabel")
        self.formLayout_5.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.minimumDelayBetweenIdentityCreationLabel)
        self.minimumDelayBetweenIdentityCreationValueLabel = QtWidgets.QLabel(self.identityGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.minimumDelayBetweenIdentityCreationValueLabel.sizePolicy().hasHeightForWidth())
        self.minimumDelayBetweenIdentityCreationValueLabel.setSizePolicy(sizePolicy)
        self.minimumDelayBetweenIdentityCreationValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.minimumDelayBetweenIdentityCreationValueLabel.setObjectName("minimumDelayBetweenIdentityCreationValueLabel")
        self.formLayout_5.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.minimumDelayBetweenIdentityCreationValueLabel)
        self.verticalLayout_2.addWidget(self.identityGroupBox)
        self.certificationGroupBox = QtWidgets.QGroupBox(self.WOTGroupBox)
        self.certificationGroupBox.setObjectName("certificationGroupBox")
        self.formLayout_4 = QtWidgets.QFormLayout(self.certificationGroupBox)
        self.formLayout_4.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_4.setObjectName("formLayout_4")
        self.certificationNumberToBeMemberLabel = QtWidgets.QLabel(self.certificationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.certificationNumberToBeMemberLabel.sizePolicy().hasHeightForWidth())
        self.certificationNumberToBeMemberLabel.setSizePolicy(sizePolicy)
        self.certificationNumberToBeMemberLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.certificationNumberToBeMemberLabel.setObjectName("certificationNumberToBeMemberLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.certificationNumberToBeMemberLabel)
        self.certificationNumberToBeMemberValueLabel = QtWidgets.QLabel(self.certificationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.certificationNumberToBeMemberValueLabel.sizePolicy().hasHeightForWidth())
        self.certificationNumberToBeMemberValueLabel.setSizePolicy(sizePolicy)
        self.certificationNumberToBeMemberValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.certificationNumberToBeMemberValueLabel.setObjectName("certificationNumberToBeMemberValueLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.certificationNumberToBeMemberValueLabel)
        self.minimumCertificationsReceivedToBeCertifierLabel = QtWidgets.QLabel(self.certificationGroupBox)
        self.minimumCertificationsReceivedToBeCertifierLabel.setObjectName("minimumCertificationsReceivedToBeCertifierLabel")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.minimumCertificationsReceivedToBeCertifierLabel)
        self.minimumCertificationsReceivedToBeCertifierValueLabel = QtWidgets.QLabel(self.certificationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.minimumCertificationsReceivedToBeCertifierValueLabel.sizePolicy().hasHeightForWidth())
        self.minimumCertificationsReceivedToBeCertifierValueLabel.setSizePolicy(sizePolicy)
        self.minimumCertificationsReceivedToBeCertifierValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.minimumCertificationsReceivedToBeCertifierValueLabel.setObjectName("minimumCertificationsReceivedToBeCertifierValueLabel")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.minimumCertificationsReceivedToBeCertifierValueLabel)
        self.maximumNumberOfCertificationPerMemberLabel = QtWidgets.QLabel(self.certificationGroupBox)
        self.maximumNumberOfCertificationPerMemberLabel.setObjectName("maximumNumberOfCertificationPerMemberLabel")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.maximumNumberOfCertificationPerMemberLabel)
        self.maximumNumberOfCertificationPerMemberValueLabel = QtWidgets.QLabel(self.certificationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.maximumNumberOfCertificationPerMemberValueLabel.sizePolicy().hasHeightForWidth())
        self.maximumNumberOfCertificationPerMemberValueLabel.setSizePolicy(sizePolicy)
        self.maximumNumberOfCertificationPerMemberValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.maximumNumberOfCertificationPerMemberValueLabel.setObjectName("maximumNumberOfCertificationPerMemberValueLabel")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.maximumNumberOfCertificationPerMemberValueLabel)
        self.minimumDelayBetweenTwoCertificationsLabel = QtWidgets.QLabel(self.certificationGroupBox)
        self.minimumDelayBetweenTwoCertificationsLabel.setObjectName("minimumDelayBetweenTwoCertificationsLabel")
        self.formLayout_4.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.minimumDelayBetweenTwoCertificationsLabel)
        self.validityPeriodOfCertificationLabel = QtWidgets.QLabel(self.certificationGroupBox)
        self.validityPeriodOfCertificationLabel.setObjectName("validityPeriodOfCertificationLabel")
        self.formLayout_4.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.validityPeriodOfCertificationLabel)
        self.validityPeriodOfCertificationValueLabel = QtWidgets.QLabel(self.certificationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.validityPeriodOfCertificationValueLabel.sizePolicy().hasHeightForWidth())
        self.validityPeriodOfCertificationValueLabel.setSizePolicy(sizePolicy)
        self.validityPeriodOfCertificationValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.validityPeriodOfCertificationValueLabel.setObjectName("validityPeriodOfCertificationValueLabel")
        self.formLayout_4.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.validityPeriodOfCertificationValueLabel)
        self.minimumDelayBetweenTwoCertificationsValueLabel = QtWidgets.QLabel(self.certificationGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.minimumDelayBetweenTwoCertificationsValueLabel.sizePolicy().hasHeightForWidth())
        self.minimumDelayBetweenTwoCertificationsValueLabel.setSizePolicy(sizePolicy)
        self.minimumDelayBetweenTwoCertificationsValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.minimumDelayBetweenTwoCertificationsValueLabel.setObjectName("minimumDelayBetweenTwoCertificationsValueLabel")
        self.formLayout_4.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.minimumDelayBetweenTwoCertificationsValueLabel)
        self.verticalLayout_2.addWidget(self.certificationGroupBox)
        self.membershipGroupBox = QtWidgets.QGroupBox(self.WOTGroupBox)
        self.membershipGroupBox.setObjectName("membershipGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.membershipGroupBox)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.minimumDelayBetweenTwoMembershipRenewalsLabel = QtWidgets.QLabel(self.membershipGroupBox)
        self.minimumDelayBetweenTwoMembershipRenewalsLabel.setObjectName("minimumDelayBetweenTwoMembershipRenewalsLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.minimumDelayBetweenTwoMembershipRenewalsLabel)
        self.minimumDelayBetweenTwoMembershipRenewalsValueLabel = QtWidgets.QLabel(self.membershipGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.minimumDelayBetweenTwoMembershipRenewalsValueLabel.sizePolicy().hasHeightForWidth())
        self.minimumDelayBetweenTwoMembershipRenewalsValueLabel.setSizePolicy(sizePolicy)
        self.minimumDelayBetweenTwoMembershipRenewalsValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.minimumDelayBetweenTwoMembershipRenewalsValueLabel.setObjectName("minimumDelayBetweenTwoMembershipRenewalsValueLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.minimumDelayBetweenTwoMembershipRenewalsValueLabel)
        self.membershipValidityPeriodLabel = QtWidgets.QLabel(self.membershipGroupBox)
        self.membershipValidityPeriodLabel.setObjectName("membershipValidityPeriodLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.membershipValidityPeriodLabel)
        self.membershipValidityPeriodValueLabel = QtWidgets.QLabel(self.membershipGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.membershipValidityPeriodValueLabel.sizePolicy().hasHeightForWidth())
        self.membershipValidityPeriodValueLabel.setSizePolicy(sizePolicy)
        self.membershipValidityPeriodValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.membershipValidityPeriodValueLabel.setObjectName("membershipValidityPeriodValueLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.membershipValidityPeriodValueLabel)
        self.maximumDistanceInStepLabel = QtWidgets.QLabel(self.membershipGroupBox)
        self.maximumDistanceInStepLabel.setObjectName("maximumDistanceInStepLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.maximumDistanceInStepLabel)
        self.maximumDistanceInStepValueLabel = QtWidgets.QLabel(self.membershipGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.maximumDistanceInStepValueLabel.sizePolicy().hasHeightForWidth())
        self.maximumDistanceInStepValueLabel.setSizePolicy(sizePolicy)
        self.maximumDistanceInStepValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.maximumDistanceInStepValueLabel.setObjectName("maximumDistanceInStepValueLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.maximumDistanceInStepValueLabel)
        self.minimumPercentageOfRemoteReferralMembersToBeMemberLabel = QtWidgets.QLabel(self.membershipGroupBox)
        self.minimumPercentageOfRemoteReferralMembersToBeMemberLabel.setObjectName("minimumPercentageOfRemoteReferralMembersToBeMemberLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.minimumPercentageOfRemoteReferralMembersToBeMemberLabel)
        self.minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel = QtWidgets.QLabel(self.membershipGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel.sizePolicy().hasHeightForWidth())
        self.minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel.setSizePolicy(sizePolicy)
        self.minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel.setObjectName("minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel)
        self.verticalLayout_2.addWidget(self.membershipGroupBox)
        self.smithGroupBox = QtWidgets.QGroupBox(self.WOTGroupBox)
        self.smithGroupBox.setObjectName("smithGroupBox")
        self.formLayout_6 = QtWidgets.QFormLayout(self.smithGroupBox)
        self.formLayout_6.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_6.setObjectName("formLayout_6")
        self.maximumCertificationsPerSmithLabel = QtWidgets.QLabel(self.smithGroupBox)
        self.maximumCertificationsPerSmithLabel.setObjectName("maximumCertificationsPerSmithLabel")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.maximumCertificationsPerSmithLabel)
        self.maximumCertificationsPerSmithValueLabel = QtWidgets.QLabel(self.smithGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.maximumCertificationsPerSmithValueLabel.sizePolicy().hasHeightForWidth())
        self.maximumCertificationsPerSmithValueLabel.setSizePolicy(sizePolicy)
        self.maximumCertificationsPerSmithValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.maximumCertificationsPerSmithValueLabel.setObjectName("maximumCertificationsPerSmithValueLabel")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.maximumCertificationsPerSmithValueLabel)
        self.numberOfCertificationsToBecomeSmithLabel = QtWidgets.QLabel(self.smithGroupBox)
        self.numberOfCertificationsToBecomeSmithLabel.setObjectName("numberOfCertificationsToBecomeSmithLabel")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.numberOfCertificationsToBecomeSmithLabel)
        self.numberOfCertificationsToBecomeSmithValueLabel = QtWidgets.QLabel(self.smithGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.numberOfCertificationsToBecomeSmithValueLabel.sizePolicy().hasHeightForWidth())
        self.numberOfCertificationsToBecomeSmithValueLabel.setSizePolicy(sizePolicy)
        self.numberOfCertificationsToBecomeSmithValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.numberOfCertificationsToBecomeSmithValueLabel.setObjectName("numberOfCertificationsToBecomeSmithValueLabel")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.numberOfCertificationsToBecomeSmithValueLabel)
        self.maximumInactivityDurationAllowedForSmithLabel = QtWidgets.QLabel(self.smithGroupBox)
        self.maximumInactivityDurationAllowedForSmithLabel.setObjectName("maximumInactivityDurationAllowedForSmithLabel")
        self.formLayout_6.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.maximumInactivityDurationAllowedForSmithLabel)
        self.maximumInactivityDurationAllowedForSmithValueLabel = QtWidgets.QLabel(self.smithGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.maximumInactivityDurationAllowedForSmithValueLabel.sizePolicy().hasHeightForWidth())
        self.maximumInactivityDurationAllowedForSmithValueLabel.setSizePolicy(sizePolicy)
        self.maximumInactivityDurationAllowedForSmithValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid rgb(34, 156, 244);\n"
"border-radius: 10%;\n"
"background: rgb(34, 156, 244);\n"
"\n"
"\n"
"")
        self.maximumInactivityDurationAllowedForSmithValueLabel.setObjectName("maximumInactivityDurationAllowedForSmithValueLabel")
        self.formLayout_6.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.maximumInactivityDurationAllowedForSmithValueLabel)
        self.verticalLayout_2.addWidget(self.smithGroupBox)
        self.verticalLayout_3.addWidget(self.WOTGroupBox)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)

        self.retranslateUi(currencyWidget)
        QtCore.QMetaObject.connectSlotsByName(currencyWidget)

    def retranslateUi(self, currencyWidget):
        #  _translate = QtCore.QCoreApplication.translate
        currencyWidget.setWindowTitle(_("Currency parameters"))
        self.refreshButton.setText(_("Refresh"))
        self.currencyGroupBox.setTitle(_("Currency"))
        self.nameLabel.setText(_("Name"))
        self.symbolLabel.setText(_("Symbol"))
        self.universalDividendLabel.setText(_("Universal Dividend"))
        self.universalDividendValueLabel.setText(_("?"))
        self.monetaryMassLabel.setText(_("Monetary Mass"))
        self.monetaryMassValueLabel.setText(_("?"))
        self.membersLabel.setText(_("Members"))
        self.membersValueLabel.setText(_("?"))
        self.blockchainGroupBox.setTitle(_("Blockchain"))
        self.expectedBlockDurationLabel.setText(_("Expected Block duration"))
        self.expectedBlockDurationValueLabel.setText(_("?"))
        self.expectedEpochDurationLabel.setText(_("Expected Epoch duration"))
        self.expectedEpochDurationValueLabel.setText(_("?"))
        self.WOTGroupBox.setTitle(_("Web of Trust"))
        self.identityGroupBox.setTitle(_("Identity"))
        self.automaticRevocationPeriodLabel.setText(_("Automatic revocation period after losing membership"))
        self.automaticRevocationPeriodValueLabel.setText(_("?"))
        self.minimumDelayBetweenChangingIdentityOwnerAccountLabel.setText(_("Minimum delay between changing identity owner account"))
        self.minimumDelayBetweenChangingIdentityOwnerAccountValueLabel.setText(_("?"))
        self.confirmIdentityPeriodLabel.setText(_("Confirm identity period"))
        self.confirmIdentityPeriodValueLabel.setText(_("?"))
        self.identityValidationPeriodLabel.setText(_("Identity validation period"))
        self.identityValidationPeriodValueLabel.setText(_("?"))
        self.identityDeletionAfterRevocationLabel.setText(_("Identity deletion after revocation period"))
        self.identityDeletionAfterRevocationValueLabel.setText(_("?"))
        self.minimumDelayBetweenIdentityCreationLabel.setText(_("Minimum delay between Identity creation"))
        self.minimumDelayBetweenIdentityCreationValueLabel.setText(_("?"))
        self.certificationGroupBox.setTitle(_("Certification"))
        self.certificationNumberToBeMemberLabel.setText(_("Number of certifications to become a member"))
        self.certificationNumberToBeMemberValueLabel.setText(_("?"))
        self.minimumCertificationsReceivedToBeCertifierLabel.setText(_("Minimum certifications received to be a certifier"))
        self.minimumCertificationsReceivedToBeCertifierValueLabel.setText(_("?"))
        self.maximumNumberOfCertificationPerMemberLabel.setText(_("Maximum number of certifications issued per member"))
        self.maximumNumberOfCertificationPerMemberValueLabel.setText(_("?"))
        self.minimumDelayBetweenTwoCertificationsLabel.setText(_("Minimum delay between certifications"))
        self.validityPeriodOfCertificationLabel.setText(_("Validity period of a certification"))
        self.validityPeriodOfCertificationValueLabel.setText(_("?"))
        self.minimumDelayBetweenTwoCertificationsValueLabel.setText(_("?"))
        self.membershipGroupBox.setTitle(_("Membership"))
        self.minimumDelayBetweenTwoMembershipRenewalsLabel.setText(_("Minimum delay between two membership renewals"))
        self.minimumDelayBetweenTwoMembershipRenewalsValueLabel.setText(_("?"))
        self.membershipValidityPeriodLabel.setText(_("Membership validity period"))
        self.membershipValidityPeriodValueLabel.setText(_("?"))
        self.maximumDistanceInStepLabel.setText(_("Maximum distance, through certifications, between an identity and referring members"))
        self.maximumDistanceInStepValueLabel.setText(_("?"))
        self.minimumPercentageOfRemoteReferralMembersToBeMemberLabel.setText(_("Minimum percentage of remote referral members for membership"))
        self.minimumPercentageOfRemoteReferralMembersToBeMemberValueLabel.setText(_("?"))
        self.smithGroupBox.setTitle(_("Smith"))
        self.maximumCertificationsPerSmithLabel.setText(_("Maximum number of certification issued per smith"))
        self.maximumCertificationsPerSmithValueLabel.setText(_("?"))
        self.numberOfCertificationsToBecomeSmithLabel.setText(_("Number of certifications to become a smith"))
        self.numberOfCertificationsToBecomeSmithValueLabel.setText(_("?"))
        self.maximumInactivityDurationAllowedForSmithLabel.setText(_("Maximum inactivity duration allowed to stay smith"))
        self.maximumInactivityDurationAllowedForSmithValueLabel.setText(_("?"))
import tikka.slots.pyqt.resources.icons.index_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account_transfer_row.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountTransferRowWidget(object):
    def setupUi(self, AccountTransferRowWidget):
        AccountTransferRowWidget.setObjectName("AccountTransferRowWidget")
        AccountTransferRowWidget.resize(838, 73)
        self.verticalLayout = QtWidgets.QVBoxLayout(AccountTransferRowWidget)
        self.verticalLayout.setContentsMargins(-1, 0, -1, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout.setVerticalSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.amountValueLabel = QtWidgets.QLabel(AccountTransferRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.amountValueLabel.sizePolicy().hasHeightForWidth())
        self.amountValueLabel.setSizePolicy(sizePolicy)
        self.amountValueLabel.setStyleSheet("color: white;\n"
"border : 3px solid #73AD21;\n"
"border-radius: 10%;\n"
"background: #73AD21;\n"
"\n"
"\n"
"")
        self.amountValueLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.amountValueLabel.setObjectName("amountValueLabel")
        self.horizontalLayout.addWidget(self.amountValueLabel)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 4, 1, 1)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.accountAddressValuelabel = QtWidgets.QLabel(AccountTransferRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountAddressValuelabel.sizePolicy().hasHeightForWidth())
        self.accountAddressValuelabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Courier")
        self.accountAddressValuelabel.setFont(font)
        self.accountAddressValuelabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.accountAddressValuelabel.setObjectName("accountAddressValuelabel")
        self.verticalLayout_2.addWidget(self.accountAddressValuelabel)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.identityNameValueLabel = QtWidgets.QLabel(AccountTransferRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.identityNameValueLabel.sizePolicy().hasHeightForWidth())
        self.identityNameValueLabel.setSizePolicy(sizePolicy)
        self.identityNameValueLabel.setObjectName("identityNameValueLabel")
        self.horizontalLayout_2.addWidget(self.identityNameValueLabel)
        self.accountNameValueLabel = QtWidgets.QLabel(AccountTransferRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountNameValueLabel.sizePolicy().hasHeightForWidth())
        self.accountNameValueLabel.setSizePolicy(sizePolicy)
        self.accountNameValueLabel.setObjectName("accountNameValueLabel")
        self.horizontalLayout_2.addWidget(self.accountNameValueLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.datetimeValueLabel = QtWidgets.QLabel(AccountTransferRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.datetimeValueLabel.sizePolicy().hasHeightForWidth())
        self.datetimeValueLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setItalic(True)
        self.datetimeValueLabel.setFont(font)
        self.datetimeValueLabel.setStyleSheet("color: \"gray\"")
        self.datetimeValueLabel.setObjectName("datetimeValueLabel")
        self.verticalLayout_2.addWidget(self.datetimeValueLabel)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.commentValueLabel = QtWidgets.QLabel(AccountTransferRowWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.commentValueLabel.sizePolicy().hasHeightForWidth())
        self.commentValueLabel.setSizePolicy(sizePolicy)
        self.commentValueLabel.setAlignment(QtCore.Qt.AlignHCenter|QtCore.Qt.AlignTop)
        self.commentValueLabel.setWordWrap(True)
        self.commentValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.commentValueLabel.setObjectName("commentValueLabel")
        self.gridLayout.addWidget(self.commentValueLabel, 0, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.line = QtWidgets.QFrame(AccountTransferRowWidget)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)

        self.retranslateUi(AccountTransferRowWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountTransferRowWidget)

    def retranslateUi(self, AccountTransferRowWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountTransferRowWidget.setWindowTitle(_("TransferWidget"))
        self.amountValueLabel.setText(_("0"))
        self.accountAddressValuelabel.setText(_("address"))
        self.identityNameValueLabel.setText(_("name#0000"))
        self.accountNameValueLabel.setText(_(" - Name"))
        self.datetimeValueLabel.setText(_("datetime"))
        self.commentValueLabel.setText(_("0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 "))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/v1_account_import.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_V1AccountImportDialog(object):
    def setupUi(self, V1AccountImportDialog):
        V1AccountImportDialog.setObjectName("V1AccountImportDialog")
        V1AccountImportDialog.resize(793, 456)
        self.verticalLayout = QtWidgets.QVBoxLayout(V1AccountImportDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.imagelabel = QtWidgets.QLabel(V1AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imagelabel.sizePolicy().hasHeightForWidth())
        self.imagelabel.setSizePolicy(sizePolicy)
        self.imagelabel.setMinimumSize(QtCore.QSize(0, 0))
        self.imagelabel.setMaximumSize(QtCore.QSize(200, 200))
        self.imagelabel.setSizeIncrement(QtCore.QSize(0, 0))
        self.imagelabel.setBaseSize(QtCore.QSize(0, 0))
        self.imagelabel.setText("")
        self.imagelabel.setPixmap(QtGui.QPixmap(":/images/safe_background"))
        self.imagelabel.setScaledContents(True)
        self.imagelabel.setAlignment(QtCore.Qt.AlignCenter)
        self.imagelabel.setObjectName("imagelabel")
        self.verticalLayout.addWidget(self.imagelabel, 0, QtCore.Qt.AlignHCenter)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.v1AddressLabel = QtWidgets.QLabel(V1AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.v1AddressLabel.sizePolicy().hasHeightForWidth())
        self.v1AddressLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.v1AddressLabel.setFont(font)
        self.v1AddressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.v1AddressLabel.setObjectName("v1AddressLabel")
        self.gridLayout.addWidget(self.v1AddressLabel, 2, 0, 1, 1)
        self.nameLineEdit = QtWidgets.QLineEdit(V1AccountImportDialog)
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.gridLayout.addWidget(self.nameLineEdit, 4, 2, 1, 1)
        self.addressValueLabel = QtWidgets.QLabel(V1AccountImportDialog)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.gridLayout.addWidget(self.addressValueLabel, 3, 2, 1, 1)
        self.secretIDLabel = QtWidgets.QLabel(V1AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.secretIDLabel.sizePolicy().hasHeightForWidth())
        self.secretIDLabel.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.secretIDLabel.setFont(font)
        self.secretIDLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.secretIDLabel.setObjectName("secretIDLabel")
        self.gridLayout.addWidget(self.secretIDLabel, 0, 0, 1, 1)
        self.passwordIDLabel = QtWidgets.QLabel(V1AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.passwordIDLabel.setFont(font)
        self.passwordIDLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.passwordIDLabel.setObjectName("passwordIDLabel")
        self.gridLayout.addWidget(self.passwordIDLabel, 1, 0, 1, 1)
        self.nameLabel = QtWidgets.QLabel(V1AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nameLabel.setFont(font)
        self.nameLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nameLabel.setObjectName("nameLabel")
        self.gridLayout.addWidget(self.nameLabel, 4, 0, 1, 1)
        self.v1AddressValueLabel = QtWidgets.QLabel(V1AccountImportDialog)
        self.v1AddressValueLabel.setText("")
        self.v1AddressValueLabel.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.v1AddressValueLabel.setObjectName("v1AddressValueLabel")
        self.gridLayout.addWidget(self.v1AddressValueLabel, 2, 2, 1, 1)
        self.secretIDLineEdit = QtWidgets.QLineEdit(V1AccountImportDialog)
        self.secretIDLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.secretIDLineEdit.setReadOnly(False)
        self.secretIDLineEdit.setObjectName("secretIDLineEdit")
        self.gridLayout.addWidget(self.secretIDLineEdit, 0, 2, 1, 1)
        self.addressLabel = QtWidgets.QLabel(V1AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.addressLabel.setObjectName("addressLabel")
        self.gridLayout.addWidget(self.addressLabel, 3, 0, 1, 1)
        self.passwordIDLineEdit = QtWidgets.QLineEdit(V1AccountImportDialog)
        self.passwordIDLineEdit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.passwordIDLineEdit.setObjectName("passwordIDLineEdit")
        self.gridLayout.addWidget(self.passwordIDLineEdit, 1, 2, 1, 1)
        self.showButton = QtWidgets.QPushButton(V1AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.showButton.sizePolicy().hasHeightForWidth())
        self.showButton.setSizePolicy(sizePolicy)
        self.showButton.setObjectName("showButton")
        self.gridLayout.addWidget(self.showButton, 0, 3, 1, 1, QtCore.Qt.AlignRight)
        self.verticalLayout.addLayout(self.gridLayout)
        self.errorLabel = QtWidgets.QLabel(V1AccountImportDialog)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.passwordLabel = QtWidgets.QLabel(V1AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLabel.sizePolicy().hasHeightForWidth())
        self.passwordLabel.setSizePolicy(sizePolicy)
        self.passwordLabel.setSizeIncrement(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.passwordLabel.setFont(font)
        self.passwordLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.passwordLabel.setObjectName("passwordLabel")
        self.horizontalLayout.addWidget(self.passwordLabel)
        self.passwordLineEdit = QtWidgets.QLineEdit(V1AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLineEdit.sizePolicy().hasHeightForWidth())
        self.passwordLineEdit.setSizePolicy(sizePolicy)
        self.passwordLineEdit.setMaxLength(20)
        self.passwordLineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.passwordLineEdit.setReadOnly(True)
        self.passwordLineEdit.setObjectName("passwordLineEdit")
        self.horizontalLayout.addWidget(self.passwordLineEdit)
        self.passwordChangeButton = QtWidgets.QPushButton(V1AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordChangeButton.sizePolicy().hasHeightForWidth())
        self.passwordChangeButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.passwordChangeButton.setIcon(icon)
        self.passwordChangeButton.setObjectName("passwordChangeButton")
        self.horizontalLayout.addWidget(self.passwordChangeButton)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(V1AccountImportDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(V1AccountImportDialog)
        self.buttonBox.accepted.connect(V1AccountImportDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(V1AccountImportDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(V1AccountImportDialog)
        V1AccountImportDialog.setTabOrder(self.secretIDLineEdit, self.passwordIDLineEdit)
        V1AccountImportDialog.setTabOrder(self.passwordIDLineEdit, self.nameLineEdit)
        V1AccountImportDialog.setTabOrder(self.nameLineEdit, self.passwordLineEdit)
        V1AccountImportDialog.setTabOrder(self.passwordLineEdit, self.passwordChangeButton)
        V1AccountImportDialog.setTabOrder(self.passwordChangeButton, self.showButton)

    def retranslateUi(self, V1AccountImportDialog):
        #  _translate = QtCore.QCoreApplication.translate
        V1AccountImportDialog.setWindowTitle(_("Import account"))
        self.v1AddressLabel.setText(_("V1 address"))
        self.secretIDLabel.setText(_("Secret ID"))
        self.passwordIDLabel.setText(_("Password"))
        self.nameLabel.setText(_("Name"))
        self.addressLabel.setText(_("Address"))
        self.showButton.setText(_("Show"))
        self.passwordLabel.setText(_("Password"))
        self.passwordChangeButton.setText(_("Change"))
import tikka.slots.pyqt.resources.icons.index_rc



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import annotations

import abc
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from tikka.interfaces.adapters.network.indexer.indexer import (
        NetworkIndexerInterface,
    )
    from tikka.interfaces.adapters.network.node.node import NetworkNodeInterface


class NetworkInterface(abc.ABC):
    """
    NetworkInterface class
    """

    @property
    def node(self) -> NetworkNodeInterface:
        """
        Return NetworkNodeInterface instance

        :return:
        """
        raise NotImplementedError

    @property
    def indexer(self) -> NetworkIndexerInterface:
        """
        Return NetworkIndexerInterface instance

        :return:
        """
        raise NotImplementedError


class NetworkException(Exception):
    """
    NetworkException class
    """



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from pathlib import Path
from typing import Optional

from PyQt5.QtCore import QTimer
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import (
    QApplication,
    QDialog,
    QFileDialog,
    QLineEdit,
    QMessageBox,
    QWidget,
)

from tikka.domains.application import Application
from tikka.domains.entities.account import Account, AccountCryptoType
from tikka.domains.entities.constants import DATA_PATH, WALLETS_PASSWORD_LENGTH
from tikka.domains.entities.wallet import V1FileWallet
from tikka.libs.keypair import Keypair
from tikka.libs.secret import generate_alphabetic
from tikka.slots.pyqt.entities.constants import (
    ADDRESS_MONOSPACE_FONT_NAME,
    DEBOUNCE_TIME,
    WALLET_LOAD_DEFAULT_DIRECTORY_PREFERENCES_KEY,
)
from tikka.slots.pyqt.resources.gui.windows.v1_wallet_restore_from_file_rc import (
    Ui_V1WalletRestoreFromFileDialog,
)


class V1WalletRestoreFromFileWindow(QDialog, Ui_V1WalletRestoreFromFileDialog):
    """
    V1WalletRestoreFromFileWindow class
    """

    def __init__(
        self,
        application: Application,
        account: Account,
        reset_password: bool = False,
        parent: Optional[QWidget] = None,
    ):
        """
        Init V1 wallet restore from file window

        :param application: Application instance
        :param account: Account instance
        :param reset_password: Reset password if True (default to False)
        :param parent: QWidget instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self.account = account
        self.reset_password = reset_password
        self.v1_file_wallet: Optional[V1FileWallet] = None
        self._ = self.application.translator.gettext

        # set monospace font to address fields
        monospace_font = QFont(ADDRESS_MONOSPACE_FONT_NAME)
        monospace_font.setStyleHint(QFont.Monospace)
        self.addressValueLabel.setFont(monospace_font)
        self.v1AddressValueLabel.setFont(monospace_font)

        # events
        self.showButton.clicked.connect(self._on_show_button_clicked)
        self.browseFilesButton.clicked.connect(self._on_browse_files_button_clicked)
        self.filePasswordLineEdit.textChanged.connect(
            self._on_password_line_edit_changed
        )
        self.buttonBox.rejected.connect(self.close)
        self.buttonBox.accepted.connect(self._on_accepted_button)
        self.changeButton.clicked.connect(self._generate_wallet_password)

        # debounce timer
        self.debounce_timer = QTimer()
        self.debounce_timer.timeout.connect(self._load_v1_file_wallet)

        # fill form
        self.storedPasswordFrame.hide()
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
        self.addressValueLabel.setText(self.account.address)
        self.nameValueLabel.setText(self.account.name)
        self._generate_wallet_password()

    def _generate_wallet_password(self):
        """
        Generate new password for wallet encryption in UI

        :return:
        """
        self.passwordLineEdit.setText(generate_alphabetic(WALLETS_PASSWORD_LENGTH))

    def _on_show_button_clicked(self):
        """
        Triggered when user click on show button

        :return:
        """
        if self.filePasswordLineEdit.echoMode() == QLineEdit.Password:
            self.filePasswordLineEdit.setEchoMode(QLineEdit.Normal)
            self.storedpasswordLineEdit.setEchoMode(QLineEdit.Normal)
            self.showButton.setText(self._("Hide"))
        else:
            self.filePasswordLineEdit.setEchoMode(QLineEdit.Password)
            self.storedpasswordLineEdit.setEchoMode(QLineEdit.Password)
            self.showButton.setText(self._("Show"))

    def _on_browse_files_button_clicked(self):
        """
        Triggered when user click on browse files button

        :return:
        """
        # clear path
        self.pathValueLabel.setText("")

        # open file dialog
        filepath = self.open_file_dialog()
        if filepath is not None:
            # update default dir preference
            self.application.repository.preferences.set(
                WALLET_LOAD_DEFAULT_DIRECTORY_PREFERENCES_KEY,
                str(Path(filepath).expanduser().absolute().parent),
            )
            self.pathValueLabel.setText(filepath)

    def _load_v1_file_wallet(self):
        """
        Load V1 file wallet

        :return:
        """
        # stop debounce_timer to avoid infinite loop
        if self.debounce_timer.isActive():
            self.debounce_timer.stop()

        password = self.filePasswordLineEdit.text().strip()
        try:
            self.v1_file_wallet = self.application.repository.v1_file_wallets.load(
                self.pathValueLabel.text(), password
            )
        except Exception:
            self.errorLabel.setText(self._("Unable to decrypt file!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return

        if not self.v1_file_wallet.signing_key:
            self.errorLabel.setText(self._("File password is not valid!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return

        # wallet decrypted
        keypair = Keypair.create_from_seed(
            seed_hex=self.v1_file_wallet.signing_key.seed.hex(),
            ss58_format=self.application.currencies.get_current().ss58_format,
            crypto_type=AccountCryptoType.ED25519,
        )
        self.v1AddressValueLabel.setText(
            Account(keypair.ss58_address).get_v1_address(
                self.application.currencies.get_current().ss58_format
            )
        )

        # if file address is not account address...
        if keypair.ss58_address != self.account.address:
            self.errorLabel.setText(self._("File address is not the account address!"))
            self.buttonBox.button(self.buttonBox.Ok).setEnabled(False)
            return

        if (
            self.application.passwords.exists(keypair.ss58_address)
            and self.reset_password is False
        ):
            stored_password = self.application.passwords.get_clear_password(keypair)
            self.storedpasswordLineEdit.setText(stored_password)
            self.storedPasswordFrame.show()
            self.passwordFrame.hide()
        else:
            self.storedPasswordFrame.hide()
            self.passwordFrame.show()

        self.errorLabel.setText("")
        self.buttonBox.button(self.buttonBox.Ok).setEnabled(True)

    def open_file_dialog(self) -> Optional[str]:
        """
        Open file dialog and return the selected filepath or None

        :return:
        """
        default_dir = self.application.repository.preferences.get(
            WALLET_LOAD_DEFAULT_DIRECTORY_PREFERENCES_KEY
        )
        if default_dir is not None:
            default_dir = str(Path(default_dir).expanduser().absolute())
        else:
            default_dir = ""

        result = QFileDialog.getOpenFileName(
            self, self._("Choose wallet file"), default_dir, "EWIF Files (*.dunikey)"
        )
        if result[0] == "":
            return None

        return result[0]

    def _on_password_line_edit_changed(self):
        """
        Triggered when text is changed in the password field

        :return:
        """
        if self.debounce_timer.isActive():
            self.debounce_timer.stop()
        self.debounce_timer.start(DEBOUNCE_TIME)

    def _on_accepted_button(self):
        """
        Triggered when user click on cancel button

        :return:
        """
        if self.v1_file_wallet is None:
            return

        password = self.passwordLineEdit.text()
        keypair = Keypair.create_from_seed(
            seed_hex=self.v1_file_wallet.signing_key.seed.hex(),
            ss58_format=self.application.currencies.get_current().ss58_format,
            crypto_type=AccountCryptoType.ED25519,
        )
        address = keypair.ss58_address

        # if password exists for root account...
        if self.application.passwords.exists(address):
            # get stored password
            clear_password = self.application.passwords.get_clear_password(keypair)
            if clear_password is not None:
                password = clear_password
        else:
            # store new password
            self.application.passwords.new(keypair, password)

        wallet = self.application.wallets.get(address)
        if wallet is None:
            # create and store Wallet instance
            wallet = self.application.wallets.create(keypair, password)
            self.application.wallets.add(wallet)
        else:
            # display confirm dialog and get response
            button = QMessageBox.question(
                self,
                self._("Change wallet password?"),
                self._(
                    "A wallet already exists for this account. Change password for {address} wallet?"
                ).format(address=wallet.address),
            )
            if button == QMessageBox.Yes:
                # create and store Wallet instance
                new_wallet = self.application.wallets.create(keypair, password)
                self.application.wallets.update(new_wallet)

        self.account.file_import = True
        self.application.accounts.unlock(self.account, password)


if __name__ == "__main__":
    qapp = QApplication(sys.argv)
    application_ = Application(DATA_PATH)
    # credentials = test/test
    account_ = Account(
        "5GAT6CJW8yVKwUuQc7sM5Kk9GZVTpbZYk9PfjNXtvnNgAJZ1",
        name="test name",
        crypto_type=AccountCryptoType.ED25519,
        file_import=True,
    )
    V1WalletRestoreFromFileWindow(application_, account_).exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/transfers_export.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_transfersExportDialog(object):
    def setupUi(self, transfersExportDialog):
        transfersExportDialog.setObjectName("transfersExportDialog")
        transfersExportDialog.resize(801, 457)
        self.formLayout = QtWidgets.QFormLayout(transfersExportDialog)
        self.formLayout.setObjectName("formLayout")
        self.fromLabel = QtWidgets.QLabel(transfersExportDialog)
        self.fromLabel.setObjectName("fromLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.fromLabel)
        self.fromCalendarWidget = QtWidgets.QCalendarWidget(transfersExportDialog)
        self.fromCalendarWidget.setObjectName("fromCalendarWidget")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.fromCalendarWidget)
        self.toLabel = QtWidgets.QLabel(transfersExportDialog)
        self.toLabel.setObjectName("toLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.toLabel)
        self.toCalendarWidget = QtWidgets.QCalendarWidget(transfersExportDialog)
        self.toCalendarWidget.setObjectName("toCalendarWidget")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.toCalendarWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(transfersExportDialog)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.buttonBox)
        self.todayButton = QtWidgets.QPushButton(transfersExportDialog)
        self.todayButton.setObjectName("todayButton")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.todayButton)

        self.retranslateUi(transfersExportDialog)
        QtCore.QMetaObject.connectSlotsByName(transfersExportDialog)

    def retranslateUi(self, transfersExportDialog):
        #  _translate = QtCore.QCoreApplication.translate
        transfersExportDialog.setWindowTitle(_("Export as"))
        self.fromLabel.setText(_("From"))
        self.toLabel.setText(_("To"))
        self.todayButton.setText(_("Today"))



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from datetime import datetime
from typing import List, Optional

from dateutil import parser
from gql import gql

from tikka.domains.entities.transfer import Transfer
from tikka.interfaces.adapters.network.connection import NetworkConnectionError
from tikka.interfaces.adapters.network.indexer.transfers import (
    IndexerTransfersException,
    IndexerTransfersInterface,
)


class IndexerTransfers(IndexerTransfersInterface):
    """
    IndexerTransfers class
    """

    def list(
        self,
        address: str,
        limit: int,
        offset: int = 0,
        sort_column: str = IndexerTransfersInterface.COLUMN_TIMESTAMP,
        sort_order: str = IndexerTransfersInterface.SORT_ORDER_DESCENDING,
        from_datetime: Optional[datetime] = None,
        to_datetime: Optional[datetime] = None,
    ) -> List[Transfer]:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            IndexerTransfersInterface.list.__doc__
        )

        if (
            not self.indexer.connection.is_connected()
            or self.indexer.connection.client is None
        ):
            raise IndexerTransfersException(NetworkConnectionError())

        # Construire la condition de filtrage pour la date
        date_filter = []
        if from_datetime:
            date_filter.append(
                f'{{ timestamp: {{ _gte: "{from_datetime.isoformat()}" }} }}'
            )
        if to_datetime:
            date_filter.append(
                f'{{ timestamp: {{ _lte: "{to_datetime.isoformat()}" }} }}'
            )

        date_filter_str = (
            "_and: [" + ", ".join(date_filter) + "]" if date_filter else ""
        )

        # Construire la requête GraphQL
        query = gql(
            f"""
            query {{
              transfer(
                limit: {limit}
                offset: {offset}
                orderBy: {{
                    {sort_column}: {sort_order}
                }}
                where: {{
                    _or: [
                        {{ fromId: {{ _eq: \"{address}\" }} }}
                        {{ toId: {{ _eq: \"{address}\" }} }}
                    ]
                    {date_filter_str}
                }}
              ) {{
                    id
                    fromId
                    from {{
                      identity {{
                        index
                        name
                      }}
                    }}
                    toId
                    to {{
                      identity {{
                        index
                        name
                      }}
                    }}
                    amount
                    timestamp
                    comment {{
                      type
                      remark
                      remarkBytes
                    }}
                }}
            }}
        """
        )

        try:
            result = self.indexer.connection.client.execute(query)
        except Exception as exception:
            raise IndexerTransfersException(exception)

        transfers = []
        for transfer in result["transfer"]:
            issuer_identity_index = (
                transfer["from"]["identity"]["index"]
                if transfer["from"]["identity"]
                else None
            )
            issuer_identity_name = (
                transfer["from"]["identity"]["name"]
                if transfer["from"]["identity"]
                else None
            )
            receiver_identity_index = (
                transfer["to"]["identity"]["index"]
                if transfer["to"]["identity"]
                else None
            )
            receiver_identity_name = (
                transfer["to"]["identity"]["name"]
                if transfer["to"]["identity"]
                else None
            )

            comment = None
            comment_type = None
            if transfer["comment"]:
                comment = (
                    transfer["comment"]["remarkBytes"]
                    if transfer["comment"]["type"] == "RAW"
                    else transfer["comment"]["remark"]
                )
                comment_type = transfer["comment"]["type"]

            transfers.append(
                Transfer(
                    id=transfer["id"],
                    issuer_address=transfer["fromId"],
                    issuer_identity_index=issuer_identity_index,
                    issuer_identity_name=issuer_identity_name,
                    receiver_address=transfer["toId"],
                    receiver_identity_index=receiver_identity_index,
                    receiver_identity_name=receiver_identity_name,
                    amount=transfer["amount"],
                    timestamp=parser.parse(transfer["timestamp"]),
                    comment=comment,
                    comment_type=comment_type,
                )
            )

        return transfers

    def count(self, address) -> int:
        __doc__ = (  # pylint: disable=redefined-builtin, unused-variable
            IndexerTransfersInterface.count.__doc__
        )
        if (
            not self.indexer.connection.is_connected()
            or self.indexer.connection.client is None
        ):
            raise IndexerTransfersException(NetworkConnectionError())

        query = gql(
            """
            query {
                transferAggregate (
                    where: {
                        _or: [
                        {fromId: {_eq: \""""
            + address
            + """\"}}
                        {toId: {_eq: \""""
            + address
            + """\"}}
                        ]
                    }
                ) {
                    aggregate {
                        count
                    }
                }
            }
            """
        )
        try:
            result = self.indexer.connection.client.execute(query)
        except Exception as exception:
            raise IndexerTransfersException(exception)

        # {
        #     "transferAggregate": {
        #       "aggregate": {
        #         "count": 3
        #       }
        #     }
        # }
        return result["transferAggregate"]["aggregate"]["count"]



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import Optional

from PyQt5.QtCore import QMutex
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QApplication, QMainWindow, QWidget

from tikka.domains.application import Application
from tikka.domains.entities.constants import DATA_PATH
from tikka.domains.entities.events import (
    ConnectionsEvent,
    CurrencyEvent,
    IndexersEvent,
    NodesEvent,
)
from tikka.slots.pyqt.entities.constants import (
    ICON_NETWORK_CONNECTED,
    ICON_NETWORK_DISCONNECTED,
)
from tikka.slots.pyqt.entities.worker import AsyncQWorker
from tikka.slots.pyqt.resources.gui.widgets.connection_rc import Ui_ConnectionWidget


class ConnectionWidget(QWidget, Ui_ConnectionWidget):
    """
    ConnectionWidget class
    """

    def __init__(
        self,
        application: Application,
        mutex: QMutex,
        parent: Optional[QWidget] = None,
    ) -> None:
        """
        Init ConnectionWidget instance

        :param application: Application instance
        :param mutex: QMutex instance
        :param parent: MainWindow instance
        """
        super().__init__(parent=parent)
        self.setupUi(self)

        self.application = application
        self.mutex = mutex
        self._ = self.application.translator.gettext

        self.connected_button_text = self._("Disconnect")
        self.disconnected_button_text = self._("Connect")

        self.init_node_urls()
        self.init_indexer_urls()

        if self.application.connections.node.is_connected():
            self._on_node_connected()
        else:
            self._on_node_disconnected()

        if self.application.connections.indexer.is_connected():
            self._on_indexer_connected()
        else:
            self._on_indexer_disconnected()

        # events
        self.nodeUrlsComboBox.activated.connect(
            self.on_node_urls_combobox_index_changed
        )
        self.nodeConnectButton.clicked.connect(
            self._on_node_connect_button_clicked_event
        )
        self.refreshNodeButton.clicked.connect(
            self._on_refresh_node_button_clicked_event
        )

        self.indexerUrlsComboBox.activated.connect(
            self.on_indexer_urls_combobox_index_changed
        )
        self.indexerConnectButton.clicked.connect(
            self._on_indexer_connect_button_clicked_event
        )
        self.refreshIndexerButton.clicked.connect(
            self._on_refresh_indexer_button_clicked_event
        )

        # subscribe to application events
        self.application.event_dispatcher.add_event_listener(
            CurrencyEvent.EVENT_TYPE_CHANGED, self._on_currency_event
        )
        self.application.event_dispatcher.add_event_listener(
            NodesEvent.EVENT_TYPE_LIST_CHANGED,
            lambda event: self.init_node_urls(),
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_NODE_CONNECTED, self._on_node_connected
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_NODE_DISCONNECTED, self._on_node_disconnected
        )

        self.application.event_dispatcher.add_event_listener(
            IndexersEvent.EVENT_TYPE_LIST_CHANGED,
            lambda event: self.init_indexer_urls(),
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_INDEXER_CONNECTED, self._on_indexer_connected
        )
        self.application.event_dispatcher.add_event_listener(
            ConnectionsEvent.EVENT_TYPE_INDEXER_DISCONNECTED,
            self._on_indexer_disconnected,
        )

        ##############################
        # ASYNC METHODS
        ##############################
        # Create a QWorker object
        self.toggle_node_connection_async_qworker = AsyncQWorker(
            self.toggle_node_connection,
            self.mutex,
        )
        self.toggle_node_connection_async_qworker.finished.connect(
            self._on_finished_toggle_node_connection
        )
        self.toggle_indexer_connection_async_qworker = AsyncQWorker(
            self.toggle_indexer_connection,
            self.mutex,
        )
        self.toggle_indexer_connection_async_qworker.finished.connect(
            self._on_finished_toggle_indexer_connection
        )
        self.network_fetch_current_node_async_qworker = AsyncQWorker(
            self.fetch_node_from_network,
            self.mutex,
        )
        self.network_fetch_current_node_async_qworker.finished.connect(
            self._on_finished_fetch_node_from_network
        )

        if self.application.connections.node.is_connected():
            self.network_fetch_current_node_async_qworker.start()

        # Create a QWorker object
        self.network_fetch_current_indexer_async_qworker = AsyncQWorker(
            self.fetch_indexer_from_network,
            self.mutex,
        )
        self.network_fetch_current_indexer_async_qworker.finished.connect(
            self._on_finished_fetch_indexer_from_network
        )

        if self.application.connections.indexer.is_connected():
            self.network_fetch_current_indexer_async_qworker.start()

    def init_node_urls(self) -> None:
        """
        Init combobox with node urls

        :return:
        """
        self.nodeUrlsComboBox.clear()

        urls = [node.url for node in self.application.nodes.list()]
        self.nodeUrlsComboBox.addItems(urls)
        # get current node url from domain
        current_node_url = self.application.nodes.get_current_url()
        if current_node_url in urls:
            self.nodeUrlsComboBox.setCurrentIndex(urls.index(current_node_url))

    def on_node_urls_combobox_index_changed(self):
        """
        Triggered when node url selection is changed

        :return:
        """
        url = self.nodeUrlsComboBox.currentText()
        if url:
            node = self.application.nodes.get(url)
            if node is None:
                # get the first one
                url = self.nodeUrlsComboBox.itemText(0)
            self.application.nodes.set_current_url(url)

            if self.application.connections.node.is_connected():
                self.application.event_dispatcher.dispatch_event(
                    ConnectionsEvent(ConnectionsEvent.EVENT_TYPE_NODE_CONNECTED)
                )
                self.refreshNodeButton.setEnabled(False)
                self.network_fetch_current_node_async_qworker.start()
            else:
                self.application.event_dispatcher.dispatch_event(
                    ConnectionsEvent(ConnectionsEvent.EVENT_TYPE_NODE_DISCONNECTED)
                )

    def _on_node_connect_button_clicked_event(self):
        """
        Triggered when user click on connect button

        :return:
        """
        # Disable button
        self.nodeConnectButton.setDisabled(True)
        self.toggle_node_connection_async_qworker.start()

    def toggle_node_connection(self):
        """
        Toggle node connection

        :return:
        """
        if self.application.connections.node.is_connected():
            self.application.connections.node.disconnect()
            self.application.event_dispatcher.dispatch_event(
                ConnectionsEvent(ConnectionsEvent.EVENT_TYPE_NODE_DISCONNECTED)
            )
        else:
            url = self.nodeUrlsComboBox.currentText()
            if url:
                node = self.application.nodes.get(url)
                if node is not None:
                    self.application.connections.node.connect(node)

                    if self.application.connections.node.is_connected():
                        self.application.event_dispatcher.dispatch_event(
                            ConnectionsEvent(ConnectionsEvent.EVENT_TYPE_NODE_CONNECTED)
                        )

    def _on_finished_toggle_node_connection(self):
        """
        Triggered when toggle_node_connection is finished

        :return:
        """
        self.nodeConnectButton.setDisabled(False)

    def _on_refresh_node_button_clicked_event(self):
        """
        Triggered when user click on refresh node button

        :return:
        """
        # Disable button
        self.refreshNodeButton.setEnabled(False)
        self.network_fetch_current_node_async_qworker.start()

    def fetch_node_from_network(self):
        """
        Update node infos from current url connection

        :return:
        """
        try:
            self.application.nodes.network_fetch_current_node()
        except Exception:
            pass

    def _on_finished_fetch_node_from_network(self):
        """
        Triggered when async request fetch_node_from_network is finished

        :return:
        """
        self.refreshNodeButton.setEnabled(True)
        self._update_ui()

    def init_indexer_urls(self) -> None:
        """
        Init combobox with indexer urls

        :return:
        """
        self.indexerUrlsComboBox.clear()

        urls = [indexer.url for indexer in self.application.indexers.list()]
        self.indexerUrlsComboBox.addItems(urls)
        # get current node url from domain
        current_indexer_url = self.application.indexers.get_current_url()
        if current_indexer_url in urls:
            self.indexerUrlsComboBox.setCurrentIndex(urls.index(current_indexer_url))

    def on_indexer_urls_combobox_index_changed(self):
        """
        Triggered when indexer url selection is changed

        :return:
        """
        url = self.indexerUrlsComboBox.currentText()
        if url:
            indexer = self.application.indexers.get(url)
            if indexer is None:
                # get the first one
                url = self.indexerUrlsComboBox.itemText(0)
            self.application.indexers.set_current_url(url)
            if self.application.connections.indexer.is_connected():
                self.application.event_dispatcher.dispatch_event(
                    ConnectionsEvent(ConnectionsEvent.EVENT_TYPE_INDEXER_CONNECTED)
                )
                self.refreshIndexerButton.setEnabled(False)
                self.network_fetch_current_indexer_async_qworker.start()
            else:
                self.application.event_dispatcher.dispatch_event(
                    ConnectionsEvent(ConnectionsEvent.EVENT_TYPE_INDEXER_DISCONNECTED)
                )

    def _on_indexer_connect_button_clicked_event(self):
        """
        Triggered when user click on indexer connect button

        :return:
        """
        self.indexerConnectButton.setDisabled(True)
        self.toggle_indexer_connection_async_qworker.start()

    def toggle_indexer_connection(self):
        """
        Toggle indexer connection

        :return:
        """
        if self.application.connections.indexer.is_connected():
            self.application.connections.indexer.disconnect()
            self.application.event_dispatcher.dispatch_event(
                ConnectionsEvent(ConnectionsEvent.EVENT_TYPE_INDEXER_DISCONNECTED)
            )
        else:
            url = self.indexerUrlsComboBox.currentText()
            if url:
                indexer = self.application.indexers.get(url)
                if indexer is not None:
                    self.application.connections.indexer.connect(indexer)
                    if self.application.connections.indexer.is_connected():
                        self.application.event_dispatcher.dispatch_event(
                            ConnectionsEvent(
                                ConnectionsEvent.EVENT_TYPE_INDEXER_CONNECTED
                            )
                        )

    def _on_finished_toggle_indexer_connection(self):
        """
        Triggered when toggle_indexer_connection is finished

        :return:
        """
        self.indexerConnectButton.setDisabled(False)

    def _on_refresh_indexer_button_clicked_event(self):
        """
        Triggered when user click on refresh indexer button

        :return:
        """
        # Disable button
        self.refreshIndexerButton.setEnabled(False)
        self.network_fetch_current_indexer_async_qworker.start()

    def fetch_indexer_from_network(self):
        """
        Update indexer infos from current url connection

        :return:
        """
        try:
            self.application.indexers.network_fetch_current_indexer()
        except Exception:
            pass

    def _on_finished_fetch_indexer_from_network(self):
        """
        Triggered when async request fetch_indexer_from_network is finished

        :return:
        """
        self.refreshIndexerButton.setEnabled(True)
        self._update_ui()

    def _update_ui(self):
        """
        Update node infos in UI

        :return:
        """
        node_url = self.nodeUrlsComboBox.currentText()
        if node_url:
            node = self.application.nodes.get(node_url)
            if node is None:
                self.softwareValueLabel.setText("")
                self.versionValueLabel.setText("")
                self.peerIDValueLabel.setText("")
                self.nodeBlockValueLabel.setText("?")
                self.epochValueLabel.setText("?")
                self.unsafeAPIExposedValueLabel.setText("")
            else:
                self.softwareValueLabel.setText(node.software)
                self.versionValueLabel.setText(node.software_version)
                self.peerIDValueLabel.setText(node.peer_id)
                self.nodeBlockValueLabel.setText(str(node.block))
                self.epochValueLabel.setText(str(node.epoch_index))
                self.unsafeAPIExposedValueLabel.setText(
                    self._("Yes") if node.unsafe_api_exposed is True else self._("No")
                )

        indexer_url = self.indexerUrlsComboBox.currentText()
        if indexer_url:
            indexer = self.application.indexers.get(indexer_url)
            if indexer is None or indexer.block is None:
                self.indexerBlockValueLabel.setText("?")
            else:
                self.indexerBlockValueLabel.setText(str(indexer.block))

    def _on_currency_event(self, _):
        """
        When a currency event is triggered

        :param _: CurrencyEvent instance
        :return:
        """
        self.init_node_urls()
        self.init_indexer_urls()

    def _on_node_connected(self, _=None):
        """
        Triggered when node is connected

        :return:
        """
        self.nodeConnectButton.setText(self.connected_button_text)
        self.nodeConnectionStatusLabel.setPixmap(QPixmap(ICON_NETWORK_CONNECTED))

    def _on_node_disconnected(self, _=None):
        """
        Triggered when node is disconnected

        :return:
        """
        self.nodeConnectButton.setText(self.disconnected_button_text)
        self.nodeConnectionStatusLabel.setPixmap(QPixmap(ICON_NETWORK_DISCONNECTED))

    def _on_indexer_connected(self, _=None):
        """
        Triggered when indexer is connected

        :return:
        """
        self.indexerConnectButton.setText(self.connected_button_text)
        self.indexerConnectionStatusLabel.setPixmap(QPixmap(ICON_NETWORK_CONNECTED))

    def _on_indexer_disconnected(self, _=None):
        """
        Triggered when indexer is disconnected

        :return:
        """
        self.indexerConnectButton.setText(self.disconnected_button_text)
        self.indexerConnectionStatusLabel.setPixmap(QPixmap(ICON_NETWORK_DISCONNECTED))


if __name__ == "__main__":
    qapp = QApplication(sys.argv)

    application_ = Application(DATA_PATH)

    main_window = QMainWindow()
    main_window.show()

    main_window.setCentralWidget(ConnectionWidget(application_, QMutex(), main_window))

    sys.exit(qapp.exec_())



# Copyright 2021 Vincent Texier <vit@free.fr>
#
# This software is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import annotations

from tikka.adapters.network.indexer.indexer import NetworkIndexer
from tikka.adapters.network.node.node import NetworkNode
from tikka.interfaces.adapters.network.network import NetworkInterface


class Network(NetworkInterface):
    """
    Network class
    """

    def __init__(self) -> None:
        """
        Init Network instance

        """
        self._node = NetworkNode()
        self._indexer = NetworkIndexer()

    @property
    def node(self) -> NetworkNode:
        """
        Return NetworkNode instance

        :return:
        """
        return self._node

    @property
    def indexer(self) -> NetworkIndexer:
        """
        Return NetworkIndexer instance

        :return:
        """
        return self._indexer


class NetworkException(Exception):
    """
    NetworkException class
    """
